# python script to produce graphs of co2, temperature over 300000 years, and compute lag

import os.path
import urllib
from hashlib import sha256
import numpy as np
import matplotlib.pyplot as plt

# sources
url1 = 'ftp://ftp.ncdc.noaa.gov/pub/data/paleo/icecore/antarctica/vostok/co2nat.txt'
url2 = 'ftp://ftp.ncdc.noaa.gov/pub/data/paleo/icecore/antarctica/vostok/deutnat.txt'

# get sources if necessary
if not os.path.isfile('co2nat.txt'):
    file1 = urllib.urlretrieve(url1, 'co2nat.txt')
with open('co2nat.txt') as f:
    content1 = f.read()
hash1 = sha256(content1).hexdigest()
if hash1 != '22a688bf42651d157a974f585af00583531c88e2b3f6e90ec904ec50ab9e7f65':
    print("error! file hashes do not match:", url1)
    print("SHA2-256 hash should be: 22a688bf42651d157a974f585af00583531c88e2b3f6e90ec904ec50ab9e7f65")
    print("but is: ", hash1)
    
if not os.path.isfile('deutnat.txt'):
    file2 = urllib.urlretrieve(url2, 'deutnat.txt')
with open('deutnat.txt') as f:
    content2 = f.read()
hash2 = sha256(content2).hexdigest()
if hash2 != 'c69eca96499bece4b2f65d4e40240f52e0bfb4027383b3f2db68a1b3e36c75f6':
    print("error! file hashes do not match:", url2)
    print("SHA2-256 hash should be: c69eca96499bece4b2f65d4e40240f52e0bfb4027383b3f2db68a1b3e36c75f6")
    print("but is: ", hash2)

# parse files
d1 = dict()
linectr = 0
for line in content1.split('\n'):
    linectr += 1
    if (len(line) != 0 and line[0] == '#') or linectr < 156:
        continue
    words = line.split('\t')
    wds = [w for w in words if w != '']
    if len(wds) == 0:
        continue
    d1[int(wds[0])] = float(wds[1])

d2 = dict()
linectr = 0
for line in content2.split('\n'):
    linectr += 1
    if (len(line) != 0 and line[0] == '#') or linectr < 156:
        continue
    words = line.split('\t')
    wds = [w for w in words if w != '']
    if len(wds) != 4:
        continue
    d2[int(wds[1])] = float(wds[3])

# to numpy
data1 = np.array(list(sorted(d1.items())))
data1[...,1] -= np.mean(data1[...,1])
data1[...,1] /= np.std(data1[...,1], ddof=1)
data1[...,1] -= 2
data2 = np.array(list(sorted(d2.items())))
data2[...,1] -= np.mean(data2[...,1])
data2[...,1] /= np.std(data2[...,1], ddof=1)
data2[...,1] += 2

# plot
plt.plot(data1[...,0], data1[...,1], '-b', linewidth=2.0)
plt.plot(data2[...,0], data2[...,1], '-r', linewidth=2.0)
plt.xlabel('years ago')
plt.title('temperature (top) / atmospheric CO$_2$ content (bottom)')
plt.grid(True)
ax = plt.gca()
ax.invert_xaxis()
ax.set_yticklabels([])
#plt.savefig('temperature_co2.svg')
#plt.savefig('temperature_co2.png')
#plt.show()
#plt.axis([1850, 2020, 250, 450])
#plt.savefig('co2_relative.svg')
#plt.show()

# determine lag
time_range =  150000.0 # years
step_size = 20.0 # years
points = np.arange(0.0, time_range, step_size)
data1[...,1] += 2.0
data2[...,1] -= 2.0
co2 = np.interp(points, data1[...,0].transpose().tolist(), data1[...,1].transpose().tolist())
temp = np.interp(points, data2[...,0].transpose().tolist(), data2[...,1].transpose().tolist())

co2 = list(reversed(co2))
temp = list(reversed(temp))

length = len(co2)
maxlag = int(10000.0 / step_size)
xcorr = np.correlate(temp, co2, 'same')
xcorr /= np.std(xcorr, ddof=1)

print len(xcorr)
print len(points)
points -= max(points)/2

plt.clf()
plt.plot(points, xcorr)
plt.title('cross-correlation of temperature with CO$_2$-level')
ax = plt.gca()
ax.set_yticklabels([])
#plt.show()
#plt.savefig('xcorr.svg')
plt.savefig('xcorr.png')

lag = points[np.argmax(xcorr)]
print "CO2 lag:", -lag # i.e., shift co2 graph to the left by lag years to get the best fit

